
#
WINDOW_WIDTH = 400
WINDOW_HEIGHT = 120
IMG_SIZE = 16
LABEL_LENGTH = 120
LANGUAGE_DROPDOWN_LENGTH = "120"

#
PICTURE_WIDTH = 190
PICTURE_HEIGHT = 190
RATE_WIDTH = 30

HIERARCHY_TITLE_FONT_SIZE = 23
INFO_LABEL_WIDTH = 65
INFO_FONT_SIZE = 10
INFO_TITLE_FONT_SIZE = 18
ONE_LINE_HEIGHT = 18
FONT_TYPE = "Comic Sans MS"

COLOR_MAIN_BACKGROUND = "#73948d"
COLOR_CARD_MOVIE_BACKGROUND = "#b3d4c9"
COLOR_CARD_COLLECTOR_BACKGROUND = "#71a49b"
COLOR_CARD_BORDER_NORMAL_BACKGROUND = "#0e997a" #"#666666" #"#b3d4c9"
COLOR_CARD_BORDER_SELECTED_BACKGROUND = "#00ff00"
COLOR_CARDHOLDER_BACKGROUND = "#3f6b61"

COLOR_CARD_BORDER_MEDIA = "#b3d4c9"
COLOR_CARD_BORDER_CONTAINER = "#0e997a"

MAX_OVERLAPPED_CARDS = 5

RADIUS_CARDHOLDER = 10
RADIUS_CARD = 5
WIDTH_BORDER_CARD = 8
WIDTH_PANEL_MARGIN_TOP = 5
WIDTH_PANEL_MARGIN_BOTTOM = 5
WIDTH_PANEL_MARGIN_RIGHT = 5
WIDTH_PANEL_MARGIN_LEFT = 5

#COLOR_CARD_HOLDER_MAIN = "#73948d"

#
SECTION_RATING = 'rating'
SECTION_GENERAL = 'general'
SECTION_TITLE = 'title'
SECTION_EXTRA = 'extra'

RATING_KEY_FAVORITE = 'favorite'
RATING_KEY_NEW = 'new'
RATING_KEY_BEST = 'best'

FILE_CARD_INI = 'card.ini'

IMG_CONFIG_BUTTON = "cogwheel-button.png"
IMG_BACK_BUTTON = "back-button.png"
IMG_FOLDER_BUTTON = "folder.png"
IMG_EMPTY_BUTTON = "empty.png"
IMG_BEST_ON = "best-on.png"
IMG_BEST_OFF = "best-off.png"
IMG_NEW_ON = "new-on.png"
IMG_NEW_OFF = "new-off.png"
IMG_FAVORITE_ON = "favorite-on.png"
IMG_FAVORITE_OFF = "favorite-off.png"
